/**************************************************************************\
 *
 *  This file is part of a set of example programs for the Coin library.
 *  Copyright (C) 2000-2003 by Systems in Motion. All rights reserved.
 *
 *                   <URL:http://www.coin3d.org>
 *
 *  This sourcecode can be redistributed and/or modified under the
 *  terms of the GNU General Public License version 2 as published by
 *  the Free Software Foundation. See the file COPYING at the root
 *  directory of the distribution for more details.
 *
 *  As a special exception, all sourcecode of the demo examples can be
 *  used for any purpose for licensees of the Coin Professional
 *  Edition License, without the restrictions of the GNU GPL. See our
 *  web pages for information about how to acquire a Professional Edition
 *  License.
 *
 *  Systems in Motion, <URL:http://www.sim.no>, <mailto:support@sim.no>
 *
\**************************************************************************/

#include <Inventor/Win/SoWin.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/draggers/SoTranslate2Dragger.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPickStyle.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoTransform.h>

#include <Inventor/Win/SoWinRenderArea.h>

// *************************************************************************
const int  xmax = 10;
const int  ymax = 10;

const static int  faces = (xmax*ymax);

// Colors for the 12 faces
static float colors[faces][3];


// Construct a plane 
SoSeparator* makePlaneCell(SbColor color, SbVec3f position)
{
  //para cada plano manten separada sus propiedades
  SoSeparator * sub = new SoSeparator;
  //crea sus materiales y colores
  SoMaterial * mat = new SoMaterial;
  sub->addChild(mat);  
  mat->diffuseColor.setValue(color);

  //mueve los vertices a sus pociociones correspondientes	
  SoTranslation * trans = new SoTranslation;
  sub->addChild(trans);
  trans->translation = position;	
  SoCoordinate3 * coords = new SoCoordinate3;
  sub->addChild(coords);
  coords->point.set1Value(0, SbVec3f(-3, -3, 0));
  coords->point.set1Value(1, SbVec3f(3, -3, 0));
  coords->point.set1Value(2, SbVec3f(3, 3, 0));
  coords->point.set1Value(3, SbVec3f(-3, 3, 0));
  
  //y por ultimo crea las caras y regresa la celda plana lista
  SoFaceSet * face = new SoFaceSet;
  sub->addChild(face);
  face->numVertices.setValue(4);

  return sub;
}

// *************************************************************************

int main(int /* argc */,char ** argv)
{
  HWND window = SoWin::init(argv[0]);

  SoGroup * root = new SoGroup;
  root->ref();
	
  //create the camera, this camera dont have any perspective
  SoOrthographicCamera *camera = new SoOrthographicCamera;
  root->addChild(camera);
  // Create a light
  root->addChild(new SoDirectionalLight);

  //for now this are the positions of the cells
  const SbVec3f startpos[] = {SbVec3f(0, 0, 0), SbVec3f(10, 0, 2), SbVec3f(5, 10, 4)};
  //an the colors
  const SbColor colors[] = {SbColor(1, 0, 0), SbColor(0, 1, 0), SbColor(0, 0, 1)};
  
  //create the cells
  for (int i = 0; i < 3; i++) {
    root->addChild(makePlaneCell(colors[i], startpos[i]));
  }

  //use the simplest render area
  SoWinRenderArea *myRenderArea = new SoWinRenderArea(window);
  //set thge camera to view all the elements
  SbViewportRegion myRegion(myRenderArea->getSize());
  camera->viewAll(root, myRegion);
   
  myRenderArea->setSceneGraph(root);
  myRenderArea->setTitle("primeras celdas");
  myRenderArea->show();

  SoWin::show(window);
  SoWin::mainLoop();

  root->unref();

  return 0;
} // main()

// *************************************************************************
